﻿using APILOL.Mapper;
using ManagersEF;
using Model;
using System.Data.SqlTypes;
using System.Linq;

namespace ManagersEF
{
    public partial class EFManager
    {
        public class RunePagesManager : IRunePagesManager
        {
            private readonly EFManager parent;

            public RunePagesManager(EFManager parent)
                => this.parent = parent;

            public async Task<RunePage?> AddItem(RunePage? item)
            {
                await parent.DbContext.RunePage.AddAsync(item.ToEntity());
                return item;
            }

            public async Task<bool> DeleteItem(RunePage? item)
            {
                parent.DbContext.RunePage.Remove(item.ToEntity());
                return true;
            }

            public async Task<IEnumerable<RunePage?>> GetItems(int index, int count, string? orderingPropertyName = null, bool descending = false)
            {
                return parent.DbContext.RunePage.GetItemsWithFilterAndOrdering(
                        rp => true,
                        index, count,
                        orderingPropertyName, descending).Select(rp => rp.ToModel());
            }

            public async Task<IEnumerable<RunePage?>> GetItemsByChampion(Champion? champion, int index, int count, string? orderingPropertyName = null, bool descending = false)
            {
                return parent.DbContext.RunePage.GetItemsWithFilterAndOrdering(
                    rp => rp.Champions.Any(c => c.Name.Equals(champion.Name)),
                    index, count,
                    orderingPropertyName, descending).Select(rp => rp.ToModel());
            }

            public async Task<IEnumerable<RunePage?>> GetItemsByName(string substring, int index, int count, string? orderingPropertyName = null, bool descending = false)
            {
                return parent.DbContext.RunePage.GetItemsWithFilterAndOrdering(
                    rp => rp.Name.Contains(substring),
                    index, count,
                    orderingPropertyName, descending).Select(rp => rp.ToModel());
            }

            public async Task<IEnumerable<RunePage?>> GetItemsByRune(Rune? rune, int index, int count, string? orderingPropertyName = null, bool descending = false)
            {
                return parent.DbContext.RunePage.GetItemsWithFilterAndOrdering(
                    rp => rp.Runes.Any(r => r.Name.Equals(rune.Name)),
                    index, count,
                    orderingPropertyName, descending).Select(rp => rp.ToModel());
            }

            public async Task<int> GetNbItems()
            {
                return parent.DbContext.RunePage.Count();

            }

            public async Task<int> GetNbItemsByChampion(Champion? champion)
            {
                return parent.DbContext.RunePage.Where(rp => rp.Champions.Any(c => c.Name.Equals(champion.Name))).Count();

            }

            public async Task<int> GetNbItemsByName(string substring)
            {
                return parent.DbContext.RunePage.Where(rp => rp.Name.Contains(substring)).Count();
            }

            public async Task<int> GetNbItemsByRune(Model.Rune? rune)
            {
                return parent.DbContext.RunePage.Where(rp => rp.Runes.Any(r => r.Name.Equals(rune.Name))).Count();
            }

            public async Task<RunePage?> UpdateItem(RunePage? oldItem, RunePage? newItem)
            {
                parent.DbContext.RunePage.Remove(oldItem.ToEntity());
                parent.DbContext.RunePage.Add(newItem.ToEntity());
                return newItem;
            }
        }
    }
}